import utime as time
import uasyncio as asyncio
import machine

from configure import *
from lcd_i2c_printer import *
from urbanbus import bus_service
        
text_added      = 'Powered by welbits.com'

# Read de configuration files                       
config          = read_config_file()
config_client   = config['configuration']['network']['client']
stopid          = config['settings']['stopid']

# Boot message sequence
name  = config['about']['name']
model = config['about']['model']

print_on_lcd(name)
time.sleep(1)
print_on_lcd(name, 'Model: '+ model)
time.sleep(2)
moving_message(text_added,
               1,
               0.2,
               config['about']['name'].center(I2C_NUM_COLS),
               4.8)

# Attempting to connect network

waiting_message('Conectando Wi-Fi')

try:
    wifi_client = start_wifi(config_client['ssid'],
                             config_client['password'],
                             'client')
    
    print("Status: ",wifi_client.status())
    
    if wifi_client.status() != 3: # Trigger setup
        raise OSError('ssid is empty')
    
    else: # Allow change stop id during x time
        
        waiting_message("Elija parada")
        time.sleep(2)
        
        init_server('/save_stopid')
        task1 = waiting_for_config(60, 'IP: ' + wifi_client.ifconfig()[0])
        task2 = asyncio.start_server(serve_client, '0.0.0.0', 80)
        
        asyncio.run(asyncio.gather(task1, task2))
        
        stopid = read_config_file()['settings']['stopid']
        
except OSError: # As can't connect let entry to setup page

    wifi_connected = False
    
    while not wifi_connected:
        waiting_message('Configurando')
        
        net_name        = config['configuration']['network']['ap']['ssid']
        psswrd          = config['configuration']['network']['ap']['password']
        
        access_point    = start_wifi(net_name, psswrd, 'ap')

##################################################################33
        
        init_server('/save_wifi', access_point.scan())
        
        #           0123456789ABCDEF
        msg     = ['Configure su ',
                   'dispositivo:',
                   'Conecte a SSID' ,
                   config['about']['name'],
                   'con contrase\356a:',
                   config['configuration']['network']['ap']['password'],
                   'Navegue a:',
                   access_point.ifconfig()[0]
                  ]
               
        task1 = info_during_setup(msg)
        task2 = asyncio.start_server(serve_client, '0.0.0.0', 80)

        tasks = asyncio.gather(task1,task2)
            
        asyncio.run(tasks)                        

        waiting_message('Conectando Wi-Fi')        
 #####################################################################       
        print(access_point.ifconfig())   

        time.sleep(3)
   
        access_point.active(False); access_point.deinit(); del access_point
          
        config_client = read_config_file()['configuration']['network']['client']
        time.sleep(3)
        wifi_client   = start_wifi(config_client['ssid'],
                                   config_client['password'],
                                   'client')
        
        if wifi_client.status() == 3:
            wifi_connected = True
            
        wifi_client.status()
        print ('Status: ' + str(wifi_client.status()))

    stopid = read_config_file()['settings']['stopid']
    
waiting_message('Buscando datos')
     
try : #Para la fase de desarrollo
    bus_service()
    
    
finally:
    wifi_client.active(False); wifi_client.deinit() ; del wifi_client #No conecta AP en la segunda ejecución


import machine
import utime as time

from machine import I2C
from lcd_api import LcdApi
from pico_i2c_lcd import I2cLcd


I2C_ADDR     = 0x27
I2C_NUM_ROWS = 2
I2C_NUM_COLS = 16

SDA = machine.Pin(8)
SCL = machine.Pin(9)

i2c = I2C(0, sda=machine.Pin(8), scl=machine.Pin(9), freq=400000)
lcd = I2cLcd(i2c, I2C_ADDR, I2C_NUM_ROWS, I2C_NUM_COLS)


def loop(list): # Iter lists in eternal way
    while True:
        for item in list:
            yield item


def print_on_lcd_dumb(text): # Simplier write on LCD
    reset_waiting()
    lcd.clear()
    lcd.putstr(text)


def print_on_lcd(text1, text2 = '', centered = True): # Write by lines
    spaces_to_center = I2C_NUM_COLS
    if centered == False:
        spaces_to_center = 0

    reset_waiting()
    lcd.clear()
    lcd.move_to(0,0) 
    lcd.putstr(text1.center(spaces_to_center))
    lcd.move_to(0,1)
    lcd.putstr(text2.center(spaces_to_center))


def waiting_message(message): #Print on LCD and cursor bliking and the end
    reset_waiting()
    lcd.clear()
    lcd.putstr(message)
    lcd.blink_cursor_on()


def reset_waiting():  #Erase cursor when is blinking 
    lcd.hide_cursor()
    time.sleep(0.1)
    lcd.clear()
    

def moving_message(message1, line = 1, refresh = 0.4, message2 = '', t_expire=60):
    reset_waiting()
    lcd.clear()
    if line != 0 and line !=1 :
        raise ValueError('Only 0 or 1 are allowed for line argument')
    lcd.clear()
    spaces = I2C_NUM_COLS - len(message1)
    chain_text = list(message1 + ' ' * (I2C_NUM_COLS + 1))[::-1]
    
    if line == 0:
        line2 = 1
    else:
        line2 = 0
        
    lcd.move_to(0,line2)    
    lcd.putstr(message2)
    while t_expire > 0: ### Ojo usar asíncrono o poner tope
        chtr = loop(chain_text)
        for i in range(I2C_NUM_COLS + 1):
            lcd.move_to(I2C_NUM_COLS -i,line)
            lcd.putchar(next(chtr))
            
        last = chain_text.pop()
        chain_text.insert(0, last)
        time.sleep(refresh)
        t_expire -= refresh


def insert_message(message, col, line): #Insert part movil on a string improve
    lcd.move_to(col,line)               # quality of visibility
       
    for letter in message:
        lcd.putchar(letter)
        col += 1
        lcd.move_to(col,line)       
    

def show_wait_time(arrivals, delay=60): # Optimized to show arrivals data
    matrix_lcd = []
    
    for arrival in arrivals:
        disp_spaces = I2C_NUM_COLS - (len(arrival[0]) +
                                      len(arrival[1])
                                     )
           
        matrix_lcd.append("{0}{2}{1}".format(arrival[0],
                                             arrival[1],
                                             ' ' * disp_spaces
                                             )
                          )
       
        step = loop(matrix_lcd)
        ant  = next(step)
  
    if len(matrix_lcd) > 2:
        while delay > 0:
             post = next(step)
             print_on_lcd_dumb(ant + post)
             ant = post
             time.sleep(2)
             delay -= 2
             
    elif len(matrix_lcd) == 2:
        post = next(step)
        print_on_lcd_dumb(ant + post)
        
    elif len(matrix_lcd) == 1:
        print_on_lcd_dumb(ant)
        
    else: 
        waiting_message('Configure otra  parada. Reinicie')
        
    return delay


async def info_during_setup(lines):   #Asynchronous message carrousel 
    import uasyncio as asyncio
    global finish_countdown
    from configure import finish_countdown
        
    
    def optimize(text):
        text = text + ((I2C_NUM_COLS - len(text)) * ' ')
        return text
    
    if len(lines) % 2 == 1:           #Return None    
        lines.append('')
        
    step = loop(lines)
    
    while finish_countdown == False:
        ant = next(step)               
        post = next(step)
        print_on_lcd_dumb(optimize(ant) + optimize(post))
        await asyncio.sleep(2)
        from configure import finish_countdown

        


import utime
import gc

from lcd_api import LcdApi
from machine import I2C

# PCF8574 pin definitions
MASK_RS = 0x01       # P0
MASK_RW = 0x02       # P1
MASK_E  = 0x04       # P2

SHIFT_BACKLIGHT = 3  # P3
SHIFT_DATA      = 4  # P4-P7

class I2cLcd(LcdApi):
    
    #Implements a HD44780 character LCD connected via PCF8574 on I2C

    def __init__(self, i2c, i2c_addr, num_lines, num_columns):
        self.i2c = i2c
        self.i2c_addr = i2c_addr
        self.i2c.writeto(self.i2c_addr, bytes([0]))
        utime.sleep_ms(20)   # Allow LCD time to powerup
        # Send reset 3 times
        self.hal_write_init_nibble(self.LCD_FUNCTION_RESET)
        utime.sleep_ms(5)    # Need to delay at least 4.1 msec
        self.hal_write_init_nibble(self.LCD_FUNCTION_RESET)
        utime.sleep_ms(1)
        self.hal_write_init_nibble(self.LCD_FUNCTION_RESET)
        utime.sleep_ms(1)
        # Put LCD into 4-bit mode
        self.hal_write_init_nibble(self.LCD_FUNCTION)
        utime.sleep_ms(1)
        LcdApi.__init__(self, num_lines, num_columns)
        cmd = self.LCD_FUNCTION
        if num_lines > 1:
            cmd |= self.LCD_FUNCTION_2LINES
        self.hal_write_command(cmd)
        gc.collect()

    def hal_write_init_nibble(self, nibble):
        # Writes an initialization nibble to the LCD.
        # This particular function is only used during initialization.
        byte = ((nibble >> 4) & 0x0f) << SHIFT_DATA
        self.i2c.writeto(self.i2c_addr, bytes([byte | MASK_E]))
        self.i2c.writeto(self.i2c_addr, bytes([byte]))
        gc.collect()
        
    def hal_backlight_on(self):
        # Allows the hal layer to turn the backlight on
        self.i2c.writeto(self.i2c_addr, bytes([1 << SHIFT_BACKLIGHT]))
        gc.collect()
        
    def hal_backlight_off(self):
        #Allows the hal layer to turn the backlight off
        self.i2c.writeto(self.i2c_addr, bytes([0]))
        gc.collect()
        
    def hal_write_command(self, cmd):
        # Write a command to the LCD. Data is latched on the falling edge of E.
        byte = ((self.backlight << SHIFT_BACKLIGHT) |
                (((cmd >> 4) & 0x0f) << SHIFT_DATA))
        self.i2c.writeto(self.i2c_addr, bytes([byte | MASK_E]))
        self.i2c.writeto(self.i2c_addr, bytes([byte]))
        byte = ((self.backlight << SHIFT_BACKLIGHT) |
                ((cmd & 0x0f) << SHIFT_DATA))
        self.i2c.writeto(self.i2c_addr, bytes([byte | MASK_E]))
        self.i2c.writeto(self.i2c_addr, bytes([byte]))
        if cmd <= 3:
            # The home and clear commands require a worst case delay of 4.1 msec
            utime.sleep_ms(5)
        gc.collect()

    def hal_write_data(self, data):
        # Write data to the LCD. Data is latched on the falling edge of E.
        byte = (MASK_RS |
                (self.backlight << SHIFT_BACKLIGHT) |
                (((data >> 4) & 0x0f) << SHIFT_DATA))
        self.i2c.writeto(self.i2c_addr, bytes([byte | MASK_E]))
        self.i2c.writeto(self.i2c_addr, bytes([byte]))
        byte = (MASK_RS |
                (self.backlight << SHIFT_BACKLIGHT) |
                ((data & 0x0f) << SHIFT_DATA))      
        self.i2c.writeto(self.i2c_addr, bytes([byte | MASK_E]))
        self.i2c.writeto(self.i2c_addr, bytes([byte]))
        gc.collect()




# Read the configuration on config.json file
def read_config_file():
    import ujson as json
    blank_config = """{
      "configuration": {
        "network": {
          "client": {
            "ssid": "",
            "password": ""
          },
          "ap": {
            "ssid": "WAITLESS",
            "password": "123456789"
          }
        }

      },
          "settings": {
          "transport_network": "urbanos",
          "stopid": ""
        },  
      "about": {
        "name": "WAITLESS",
        "model": "SVIZ",
        "version": "0.4.2a0"
      }
    }
    """ 
    try:
        with open('config.json') as config:
            #if config.read() != '':
            return json.loads(config.read())
            #else:
            #    raise OSError('File is empty')
    except (OSError, ValueError):
        with open('config.json','w') as config:
            config.write(blank_config)        
            return json.loads(blank_config)
        

#Starts the WiFi as client or AP    
def start_wifi(ssid, password, mode):
    import network, time
        
    if (mode == 'ap'):
        wlan = network.WLAN(network.AP_IF)
        wlan.config(essid=ssid, password=password)
        wlan.active(True)
        
    elif(mode == 'client'):
        wlan = network.WLAN(network.STA_IF)
        wlan.active(True)
        wlan.connect(ssid, password)
       
        max_wait = 10
        while max_wait > 0:
            if wlan.status() < 0 or wlan.status() >= 3:
                break
            max_wait -= 1
            print('waiting for connection...')
            time.sleep(1)

        if wlan.status() != 3:
            #raise RuntimeError('network connection failed')
            print('network connection failed')
        else:
            print('connected')
            status = wlan.ifconfig()
            print( 'ip = ' + status[0] )
            
    return wlan


#Create a radio option for an HTML form
def html_scanned_ssid(x): #A list with SSIDs scanned
    selectString = ''

    for y in x:
        html_select ="""
            <label for=\"{0}\" class=\"form-radio\">
			<input type=\"radio\" id=\"{0}\" name=\"ssid\" value=\"{0}\" required >
			<i class=\"form-icon\"></i>{0}</label>
		"""
        selectString += html_select.format(y[0].decode())

    return selectString


#Configure parameters for asynchrounous web server
def init_server(action_form,ssids = []):
    def pass_data_server():
        return action_form, ssids
    return pass_data_server


finish_countdown = False


# Asynchronous countdown optimiezed   
async def waiting_for_config(time, message1):
    import uasyncio as asyncio
    from  lcd_i2c_printer import reset_waiting, print_on_lcd, insert_message
    figures = len(str(time))
    reset_waiting()
    message2 = str(time) + ' seg'
    
    print_on_lcd(message1, message2, False)
    
    while time > -1:
        await asyncio.sleep(1)
        insert_message(' ' * (figures - len(str(time))) + str(time) + ' seg',0,1)                         
        time -= 1
        
        global finish_countdown
        
        if finish_countdown == True:
            time = -1


# Read files by chunks, let read large files 
def read_in_chunks(file_object, chunk_size=1024):
    """Lazy function (generator) to read a file piece by piece.
    Default chunk size: 1k."""
    while True:
        data = file_object.read(chunk_size)
        if not data:
            break
        yield data


# Manage the logic of web server according to the request
def manage_request(request_obj, form_action, ssids=[]):   
    request_str = str(request_obj)
    content_type = 'text/html'
    
    if request_str.find(form_action) ==7:
        update_config(request_obj, form_action)
        #global saved
        #saved = True
        response ='res/saved.html'
    
    elif '.css' in request_str:        
        ini = request_str.index('/css')
        fin = request_str.index('.css') + 4
        response = 'res' + request_str[ini : fin]
        content_type = 'text/css'
    
    elif '/finish' in request_str:
        #global saved
        #saved = True
        response = ''
        #raise RuntimeError
        
    else:
        if form_action == '/save_wifi':
            if ssids != []:
                temp_setup_html(ssids)
                response = 'tmp/tmp_setup.html'
            else:
                response = 'res/error_ssid.html'
                
        else:
            response = 'res/config.html'       
    print(saved)   
    return response, content_type


# Update the config.json file
def update_config(obj_request, form_action):
    import ujson as json
    
    number_of_lines = 2 
    
    if form_action == '/save_wifi':
        number_of_lines = 4

    
    request_str = str(obj_request)
    print(request_str)
            
    form_request = request_str.find(form_action)
    req_by_lines = obj_request.decode().split('\r\n')
    post_data = {}
    for line in (list(filter(None,req_by_lines[len(req_by_lines)-number_of_lines:len(req_by_lines)-1]))):
        #Mejorar captura de parámetros POST
        data_list = line.split('=')
        post_data[data_list[0]] = data_list[1]
        
    print(post_data)    
    prev_config_json = read_config_file()             
     
    with open('config.json','w') as config_json:
        if form_action == '/save_wifi':
            prev_config_json['configuration']['network']['client']['ssid']     = post_data['ssid']       
            prev_config_json['configuration']['network']['client']['password'] = post_data['password']        
        prev_config_json['settings']['stopid'] = post_data['stopid']
  
        config_json.write(json.dumps(prev_config_json))


# Create a html document joining a template along a form radio block whose which
# contains a list with SSIDs within the scope
def temp_setup_html(ssid_list):
    html_select = html_scanned_ssid(ssid_list)
    with open('res/setup.html') as f:
        readed_f = f.read()
        tmp_html = readed_f % html_select
        
    with open('tmp/tmp_setup.html','w') as f:
        f.write(tmp_html)


#################################################
def init_server(route,scanned_wifi=[]):
    global form_action, ssids
    form_action, ssids = route,scanned_wifi

        
async def serve_client(reader, writer):
    import uasyncio as asyncio
    import ujson as json
    import socket
    print('server running...')
    global virt_writer, saved

    virt_writer = writer   
    saved = False
    
    #action_form = data_server()[0]
    #form_action = '/save_stopid' OJO
    #form_action, ssids = load_data()
    
    request = await reader.read(1024)
    
    response, content_type = manage_request(request, form_action, ssids)
    
    if response == '':
        global finish_countdown
        finish_countdown = True
        #print(finish_countdown)
    
    writer.write('HTTP/1.0 200 OK\r\nContent-type: %s \r\n\r\n' % content_type) 

    try:
        with open(response) as f:
            for piece in read_in_chunks(f, 2048):
                writer.write(piece)
                print('chunk')
            print(response)
            
    except OSError:
        pass
    
    await writer.drain()
    await writer.wait_closed()
    
    print("Client disconnected")



import time

class LcdApi:
    
    # Implements the API for talking with HD44780 compatible character LCDs.
    # This class only knows what commands to send to the LCD, and not how to get
    # them to the LCD.
    #
    # It is expected that a derived class will implement the hal_xxx functions.
    #
    # The following constant names were lifted from the avrlib lcd.h header file,
    # with bit numbers changed to bit masks.
    
    # HD44780 LCD controller command set
    LCD_CLR             = 0x01  # DB0: clear display
    LCD_HOME            = 0x02  # DB1: return to home position

    LCD_ENTRY_MODE      = 0x04  # DB2: set entry mode
    LCD_ENTRY_INC       = 0x02  # DB1: increment
    LCD_ENTRY_SHIFT     = 0x01  # DB0: shift

    LCD_ON_CTRL         = 0x08  # DB3: turn lcd/cursor on
    LCD_ON_DISPLAY      = 0x04  # DB2: turn display on
    LCD_ON_CURSOR       = 0x02  # DB1: turn cursor on
    LCD_ON_BLINK        = 0x01  # DB0: blinking cursor

    LCD_MOVE            = 0x10  # DB4: move cursor/display
    LCD_MOVE_DISP       = 0x08  # DB3: move display (0-> move cursor)
    LCD_MOVE_RIGHT      = 0x04  # DB2: move right (0-> left)

    LCD_FUNCTION        = 0x20  # DB5: function set
    LCD_FUNCTION_8BIT   = 0x10  # DB4: set 8BIT mode (0->4BIT mode)
    LCD_FUNCTION_2LINES = 0x08  # DB3: two lines (0->one line)
    LCD_FUNCTION_10DOTS = 0x04  # DB2: 5x10 font (0->5x7 font)
    LCD_FUNCTION_RESET  = 0x30  # See "Initializing by Instruction" section

    LCD_CGRAM           = 0x40  # DB6: set CG RAM address
    LCD_DDRAM           = 0x80  # DB7: set DD RAM address

    LCD_RS_CMD          = 0
    LCD_RS_DATA         = 1

    LCD_RW_WRITE        = 0
    LCD_RW_READ         = 1

    def __init__(self, num_lines, num_columns):
        self.num_lines = num_lines
        if self.num_lines > 4:
            self.num_lines = 4
        self.num_columns = num_columns
        if self.num_columns > 40:
            self.num_columns = 40
        self.cursor_x = 0
        self.cursor_y = 0
        self.implied_newline = False
        self.backlight = True
        self.display_off()
        self.backlight_on()
        self.clear()
        self.hal_write_command(self.LCD_ENTRY_MODE | self.LCD_ENTRY_INC)
        self.hide_cursor()
        self.display_on()

    def clear(self):
        # Clears the LCD display and moves the cursor to the top left corner
        self.hal_write_command(self.LCD_CLR)
        self.hal_write_command(self.LCD_HOME)
        self.cursor_x = 0
        self.cursor_y = 0

    def show_cursor(self):
        # Causes the cursor to be made visible
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR)

    def hide_cursor(self):
        # Causes the cursor to be hidden
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY)

    def blink_cursor_on(self):
        # Turns on the cursor, and makes it blink
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR | self.LCD_ON_BLINK)

    def blink_cursor_off(self):
        # Turns on the cursor, and makes it no blink (i.e. be solid)
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR)

    def display_on(self):
        # Turns on (i.e. unblanks) the LCD
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY)

    def display_off(self):
        # Turns off (i.e. blanks) the LCD
        self.hal_write_command(self.LCD_ON_CTRL)

    def backlight_on(self):
        # Turns the backlight on.
        
        # This isn't really an LCD command, but some modules have backlight
        # controls, so this allows the hal to pass through the command.
        self.backlight = True
        self.hal_backlight_on()

    def backlight_off(self):
        # Turns the backlight off.

        # This isn't really an LCD command, but some modules have backlight
        # controls, so this allows the hal to pass through the command.
        self.backlight = False
        self.hal_backlight_off()

    def move_to(self, cursor_x, cursor_y):
        # Moves the cursor position to the indicated position. The cursor
        # position is zero based (i.e. cursor_x == 0 indicates first column).
        self.cursor_x = cursor_x
        self.cursor_y = cursor_y
        addr = cursor_x & 0x3f
        if cursor_y & 1:
            addr += 0x40    # Lines 1 & 3 add 0x40
        if cursor_y & 2:    # Lines 2 & 3 add number of columns
            addr += self.num_columns
        self.hal_write_command(self.LCD_DDRAM | addr)

    def putchar(self, char):
        # Writes the indicated character to the LCD at the current cursor
        # position, and advances the cursor by one position.
        if char == '\n':
            if self.implied_newline:
                # self.implied_newline means we advanced due to a wraparound,
                # so if we get a newline right after that we ignore it.
                pass
            else:
                self.cursor_x = self.num_columns
        else:
            self.hal_write_data(ord(char))
            self.cursor_x += 1
        if self.cursor_x >= self.num_columns:
            self.cursor_x = 0
            self.cursor_y += 1
            self.implied_newline = (char != '\n')
        if self.cursor_y >= self.num_lines:
            self.cursor_y = 0
        self.move_to(self.cursor_x, self.cursor_y)

    def putstr(self, string):
        # Write the indicated string to the LCD at the current cursor
        # position and advances the cursor position appropriately.
        for char in string:
            self.putchar(char)

    def custom_char(self, location, charmap):
        # Write a character to one of the 8 CGRAM locations, available
        # as chr(0) through chr(7).
        location &= 0x7
        self.hal_write_command(self.LCD_CGRAM | (location << 3))
        self.hal_sleep_us(40)
        for i in range(8):
            self.hal_write_data(charmap[i])
            self.hal_sleep_us(40)
        self.move_to(self.cursor_x, self.cursor_y)

    def hal_backlight_on(self):
        # Allows the hal layer to turn the backlight on.
        # If desired, a derived HAL class will implement this function.
        pass

    def hal_backlight_off(self):
        # Allows the hal layer to turn the backlight off.
        # If desired, a derived HAL class will implement this function.
        pass

    def hal_write_command(self, cmd):
        # Write a command to the LCD.
        # It is expected that a derived HAL class will implement this function.
        raise NotImplementedError

    def hal_write_data(self, data):
        # Write data to the LCD.
        # It is expected that a derived HAL class will implement this function.
        raise NotImplementedError

    def hal_sleep_us(self, usecs):
        # Sleep for some time (given in microseconds)
        time.sleep_us(usecs)


#import sys
# Remove current dir from sys.path, otherwise setuptools will peek up our
# module instead of system.
#sys.path.pop(0)
from setuptools import setup, find_packages

setup(
    name='waitless-sviz',
    packages=find_packages(),
    #packages=['waitless-sviz'],
    package_dir = {'': '.'},
    py_modules=['main',
    		'configure',
    		'lcd_i2c_printer',
    		'urbanbus',
    		'lib.pico_i2c_lcd',
    		'lib.lcd_api'], 
    #package_data = {"waitless-sviz": ['waitless-sviz.res/*',
    #					'waitless-sviz.res.css/*']
    #		   },
    include_package_data=True, 
    version='0.4.2a0',
    description='MicroPython app to monitorize time arrivals of each bus stop in Madrid Region',
    long_description='This package runs as APP for Raspberry Pi Pico. This APP monitorize each bus stop in Madrid Region (Spain) and get time of arrival estimated which is showed in a LCD 16x2',
    keywords='bus arrivals madrid micropython',
    url='https://github.com/modl-ideas/WAITLESS_SVIZ',
    author='Miguel Palomino Civantos',
    author_email='miguelpalominocivantos@gmail.com',
    maintainer='Miguel Palomino Civantos',
    maintainer_email='miguelpalominocivantos@gmail.com',
    license='MIT',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: Implementation :: MicroPython',
        'License :: OSI Approved :: MIT License',
    ],
    install_requires = [],	
    platforms = "any"#,
    #cmdclass={'sdist': sdist_upip.sdist}
)




def url(stopid): 
    url_welbits = f'https://api.interurbanos.welbits.com/v1/stop/{stopid}'    
    url_crtm    = f'https://www.crtm.es/widgets/api/GetStopsTimes.php?codStop=8_{stopid}&type=1&orderBy=2&stopTimesByIti=3'
    
    return url_crtm

#Call to API and get json data
def call_for_data(url):
    try:
        import urequests as requests
    except ImportError:       
        import requests
        
    r               = requests.get(url = url)   
    try:
        json_data   = r.json() #r.text.json()
        status_code     = r.status_code
        
    except ValueError:
        status_code = 500
        
    status_code = r.status_code
    #print('Testeo si ejecuta r.close')
    r.close()
    return json_data, status_code


def how_much_to(value):
    import utime as time
    
    h, m     = int(value[0]), int(value[1])
    now      = time.localtime()
    after    = list(now)

    if now[3] >= h:
        if now[4] >= m:
            after[2] = after[2] + 1
    after[3],after[4] = h, m
    
    return time.mktime(tuple(after)) - time.time()

def date_ticks_from_date_str(date_str):
    import utime as time
    
   #0    5    10   15   20   25#
   #:....:....:....:....:....: 
   # 2022-09-19T17:29:16+02:00
   # YYYY-MM-DDThh:mm:ss+xx:xx
   
    now = time.localtime()
    
    Y         = int(date_str[0:4])
    M         = int(date_str[5:7])
    D         = int(date_str[8:10])
    h         = int(date_str[11:13])
    m         = int(date_str[14:16])
    s         = int(date_str[17:19])
    w         = now[6]
    y         = now[7]
    
    date_time = (Y,M,D,h,m,s,w,y)
        
    return time.mktime((Y,M,D,h,m,s,w,y))
    

#Filter for the next arrival for each line
def sel_next_bus_each_line(api_data): #sel_next_bus_each_line_crtm
    
    actual_date       = api_data['stopTimes']['actualDate']
    ticks_actual_date = date_ticks_from_date_str(actual_date) 
    data              = api_data['stopTimes']['times']['Time']
    
    next_bus = [] ; control = []
    
    for i in data:
        if i['line']['shortDescription'] not in control:
            wait_time = int ((date_ticks_from_date_str(i['time']) - ticks_actual_date) / 60 )
            if wait_time < 60:
                wait_time = str(wait_time) + ' min'
            else:
                wait_time = i['time'][11:16]
            
            next_bus.append([i['line']['shortDescription'],wait_time])
            control.append(i['line']['shortDescription'])
            
            manage_time_out_of_service(next_bus)
                
    return next_bus  # output: list

def sel_next_bus_each_line_welbits(api_data):
    import utime as time
    
    next_bus = [] ; control = []
    
    for i in api_data['lines']:
        if i['lineNumber'] not in control:
            next_bus.append([i['lineNumber'],i['waitTime']])
            control.append(i['lineNumber'])
            if all(list(map(lambda a: ':' in a[1], next_bus))):
                next_bus.sort(key = lambda x: int(x[1].replace(':','')))
                first_bus_time = next_bus[0][1].split(':')
                delay = how_much_to(first_bus_time) - 30 * 60
                time.sleep(delay)
                
    return next_bus  # output: list

def manage_time_out_of_service(next_bus):
    import utime as time
    
    if all(list(map(lambda a: ':' in a[1], next_bus))):
        next_bus.sort(key = lambda x: int(x[1].replace(':','')))
        first_bus_time = next_bus[0][1].split(':')
        delay = how_much_to(first_bus_time) - 30 * 60
        time.sleep(delay)


#Manage the behaivor according to the API response
def handle_api_responses(schedule, HTTPResponseCode): # Llamar con callForData(url(stopid))
    import utime as time
    from lcd_i2c_printer import show_wait_time, waiting_message, moving_message
    
    global count_500
    break_while = False
    
    if HTTPResponseCode == 200:
        count_500 = 0
        delay = 60

        arrivals = sel_next_bus_each_line(schedule)
        delay = show_wait_time(arrivals, delay)
                       
    elif HTTPResponseCode == 404:
        print("Connection Failed. Code: " + str(HTTPResponseCode))
        moving_message('Reinicie el dispositivo y configure parada',
           1,
           0.2,
           'Parada erronea!',
           10
           )
        break_while = True
        
    elif HTTPResponseCode == 500:
        
        print("Connection Failed. Code: " + str(HTTPResponseCode))
        
        if count_500 < 10:               
            delay = 10
                
        else:
        
            waiting_message('Server Error:   Espere')
            delay = 300
        
        count_500 += 1
    
    else:
        waiting_message('Error. Reinicie')
        break_while = True
           
    time.sleep(delay)
    return break_while


#Launch the bus service
def bus_service():
    from configure import read_config_file
    
    global count_500
    count_500 = 0
    stopid = read_config_file()['settings']['stopid']
    
    while True:
        schedule, HTTPResponseCode = call_for_data(url(stopid))
        break_while = handle_api_responses(schedule, HTTPResponseCode)
        if break_while == True:
            break

